unit main;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  rmScale, rmGauge, ExtCtrls, Buttons, Counter, StdCtrls, AGraph,
  rmSevenSegDisplay, ComCtrls, LineArrow, ToolWin, ActnList, Menus, OoMisc,
  AdPort, ImgList;

type
  TfmMain = class(TForm)
    AGraph1: TAGraph;
    ApdComPort1: TApdComPort;
    Timer1: TTimer;
    MainMenu1: TMainMenu;
    mnuFiles: TMenuItem;
    mnuOpen: TMenuItem;
    mnuSaveAs: TMenuItem;
    mnuPrint: TMenuItem;
    N1: TMenuItem;
    mnuOptions: TMenuItem;
    N2: TMenuItem;
    mnuExit: TMenuItem;
    mnuAcq: TMenuItem;
    Dmarreracquisition1: TMenuItem;
    Stopperacquisition1: TMenuItem;
    ActionList1: TActionList;
    actStartAcq: TAction;
    actStopAcq: TAction;
    actOptions: TAction;
    ToolBar1: TToolBar;
    btnStartAcq: TToolButton;
    btnStopAcq: TToolButton;
    ToolButton4: TToolButton;
    btnOptions: TToolButton;
    ImageList1: TImageList;
    Panel1: TPanel;
    vuE6: TrmGauge;
    vuE1: TrmGauge;
    vuE2: TrmGauge;
    vuE5: TrmGauge;
    Label1: TLabel;
    Label3: TLabel;
    Label5: TLabel;
    Label6: TLabel;
    Label2: TLabel;
    vuE3: TrmGauge;
    vuE4: TrmGauge;
    Label4: TLabel;
    Label7: TLabel;
    Label8: TLabel;
    Label9: TLabel;
    Label10: TLabel;
    StatusBar1: TStatusBar;
    procedure FormCreate(Sender: TObject);
    procedure actStartAcqExecute(Sender: TObject);
    procedure actStopAcqExecute(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
    procedure mnuExitClick(Sender: TObject);
    procedure actOptionsExecute(Sender: TObject);
  private
    { Dclarations prives }
    TLC1543Values: array[0..13] of integer;
    AcqCount: integer;
    procedure ResetHard;
    function HardWareIsOK: boolean;
    procedure ReadTLC1543Values;
    procedure RefreshVuMeters;
    procedure AddTLC1543ValuesToGraph;
    procedure RefreshCtrlState;
  public
    { Dclarations publiques }
    procedure WriteAddr(state: boolean);
    procedure WriteCLK(state: boolean);
    function ReadDataBit: boolean;
    function ReadEOC: boolean;
  end;

var
  fmMain: TfmMain;
  AppPath: string;
  IniFileName: string;
  AcqRunning: boolean;

implementation

uses
  rmMisc, Options;

{$R *.DFM}

procedure TfmMain.FormCreate(Sender: TObject);
begin
  // initialisations diverses
  AppPath := extractFilePath(application.exeName);
  IniFileName := changeFileExt(application.exeName, '.ini');


  with AGraph1 do begin
    VertUnit := '';
    MaxItems := width - LeftMargin - RightMargin;
  end;
  ResetHard;
end;

procedure TfmMain.WriteAddr(state: boolean);
begin
  ApdComPort1.RTS := not state;
end;

procedure TfmMain.WriteCLK(state: boolean);
begin
  ApdComPort1.DTR := not state;
end;

function TfmMain.ReadDataBit: boolean;
begin
  result := not apdComPort1.CTS;
end;

function TfmMain.ReadEOC: boolean;
begin
  result := not apdComPort1.DSR;
end;

procedure TfmMain.ResetHard;
begin
  apdComPort1.Output := 'sssss';
  WriteAddr(false);
  WriteCLK(false);
  Delay(500, true);
end;

procedure TfmMain.ReadTLC1543Values;
var
  i, j, temp: integer;
  addr: string[4];
  val: integer;
  EOC: boolean;
  timeOut: integer;
  t: longInt;
begin
  timeOut := 20;

  for i := 0 to 13 do begin

    // adresse de l'entre  lire
    addr := byteToBits(i, 4);

    // rcupration des valeurs chantillonnes par le TLC1543
    temp := 1024;
    val := 0;
    for j := 1 to 10 do begin
      temp := temp div 2;
      if j < 5 then
        WriteAddr(copy(Addr, j, 1) = '1')                 // via RTS
      else
        WriteAddr(false);                                 // via RTS
      WriteCLK(true);  // front montant de l'horloge      // via DTR
      if ReadDataBit then val := val + temp;              // via CTS
      WriteCLK(false); // front descendant de l'horloge   // via DTR
    end;

    t := GetTickCount;
    repeat
      EOC := ReadEOC;                                      // via DSR
    until EOC or (GetTickCount > (t + timeOut));

    // stockage dans le tableau des valeurs
    case i of
      0 : TLC1543Values[13] := val;
      1..13 : TLC1543Values[i - 1] := val;
    end;

  end;
end;

function TfmMain.HardWareIsOK: boolean;
begin
  if AcqCount < 3 then begin
    result := true;
    exit;
  end else begin
    result :=
    (TLC1543Values[11] > 412) and (TLC1543Values[11] < 612) and
    (TLC1543Values[12] >= 0) and (TLC1543Values[12] < 100) and
    (TLC1543Values[13] > 923) and (TLC1543Values[13] <= 1023);
  end;
end;

procedure TfmMain.RefreshVuMeters;
var
  val: integer;
begin
  val := trunc(TLC1543Values[0] * (100 / 1024));
  vuE1.Position := val;
  val := trunc(TLC1543Values[1] * (100 / 1024));
  vuE2.Position := val;
  val := trunc(TLC1543Values[2] * (100 / 1024));
  vuE3.Position := val;
  val := trunc(TLC1543Values[3] * (100 / 1024));
  vuE4.Position := val;
  val := trunc(TLC1543Values[4] * (100 / 1024));
  vuE5.Position := val;
  val := trunc(TLC1543Values[5] * (100 / 1024));
  vuE6.Position := val;
end;

procedure TfmMain.AddTLC1543ValuesToGraph;
var
  i: integer;
  s: string;
  val: real;
begin
  s := '';
  for i := 0 to 5 do begin
    val := TLC1543Values[i] * (10 / 1024);
    s := s + RealToStr(2, val) + ';';
  end;
  AGraph1.AddValues(s, true);

  {
  // affichage des valeurs dans une TListbox, pour vrif
  ListBox1.Clear;
  for i := 0 to 13 do begin
    s := byteToBits(i, 4) + ' --> ' + intToStr(TLC1543Values[i]);
    ListBox1.items.add(s);
  end;
  }
end;

procedure TfmMain.RefreshCtrlState;
begin
  timer1.Enabled := AcqRunning;
  actStartAcq.enabled := not AcqRunning;
  actStopAcq.enabled := AcqRunning;
  actOptions.Enabled := not AcqRunning;
end;

procedure TfmMain.actStartAcqExecute(Sender: TObject);
begin
  AcqRunning := true;
  StatusBar1.SimpleText := 'Initialisation...';
  AcqCount := 0;
  ResetHard;
  RefreshCtrlState;
end;

procedure TfmMain.actStopAcqExecute(Sender: TObject);
begin
  AcqRunning := false;
  StatusBar1.SimpleText := 'Acquisition stoppe';
  RefreshCtrlState;
end;

procedure TfmMain.Timer1Timer(Sender: TObject);
begin
  inc(AcqCount);
  application.processMessages;
  ReadTLC1543Values;
  if HardWareIsOK then begin
    if (acqCount > 2) then begin
      RefreshVuMeters;
      AddTLC1543ValuesToGraph;
      StatusBar1.SimpleText := 'Acquisition en cours...'
    end;
  end else begin
    actStopAcqExecute(nil);
    StatusBar1.SimpleText := 'Carte d''acquisition non dtecte !';
  end;
end;

procedure TfmMain.mnuExitClick(Sender: TObject);
begin
  close;
end;

procedure TfmMain.actOptionsExecute(Sender: TObject);
var
  frm: TfmOptions;
begin
  frm := TfmOptions.Create(self);
  try
    frm.showModal;
  finally
    frm.Free;
  end;
end;

end.
